package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

public class Consumer extends AbstractBehavior<Consumer.Message> {

    public interface Message {};

    public record Reply(int i) implements Message {  }
    public enum Error implements Message {
        EMPTY_QUEUE
    }

    public static Behavior<Message> create(ActorRef<Queue.Message> queue) {
        return Behaviors.setup(context -> new Consumer(context, queue));
    }

    private final ActorRef<Queue.Message> queue;

    private Consumer(ActorContext<Message> context, ActorRef<Queue.Message> queue) {
        super(context);
        this.queue = queue;
        this.queue.tell(new Queue.Get(this.getContext().getSelf()));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(Reply.class, this::onReply)
                .onMessage(Error.class, this::onError)
                .build();
    }
    private Behavior<Message> onError(Error msg) {
        this.getContext().getLog().info("ERROR: {}", msg.toString());
        return this;
    }
    private Behavior<Message> onReply(Reply msg) {
        this.getContext().getLog().info("Got: {}", msg.i);
        return this;
    }
}
